/* Copyright (c) 1991, Scott Hess.  All rights reserved.
 * The source and object code for this file may not be
 * redistributed except as permitted by agreement with the
 * copyright holder.
 */
#if defined(RCSID)
static char rcsid[]="$Id: AppDefaults.m,v 1.1 1992/01/20 09:53:23 scott Exp $";
#endif
/* $Log: AppDefaults.m,v $
 * Revision 1.1  1992/01/20  09:53:23  scott
 * Initial revision
 *
 */
#import "AppDefaults.h"
#import <string.h>
#import <stdlib.h>
#import <appkit/appkit.h>
//#import <appkit/defaults.h>

@implementation Application (AppDefaults)
int appDefaultsReference=0;
- (const char *)default:(const char *)defName
{
    const char *defaultValue;
    if( !defName || !defName[0]) {
	return NULL;
    }
    defaultValue=NXGetDefaultValue( [self appName], defName);
    return defaultValue;
}
-(int)defaultAsInt:(const char *)defName
{
    return [self default:defName] ? atoi( [self default:defName]) :0;
}
-(BOOL)defaultAsBOOL:(const char *)defName
{
    const char *def=[self default:defName];
    if( !def) {
	return NO;
    } else if( !strcasecmp( def, "YES")) {
	return YES;
    } else if( !strcasecmp( def, "NO")) {
	return NO;
    } else {
	return atoi( def);
    }
}
-(float)defaultAsFloat:(const char *)defName
{
    return [self default:defName] ? atof( [self default:defName]) : 0.0;
}
-(NXColor)defaultAsColor:(const char *)defName
{
    const char *colorText=[self default:defName];
    if( colorText) {
	float r, g, b, a=NX_NOALPHA;
	if( sscanf( colorText, "%f %f %f %f", &r, &g, &b, &a)>=3) {
	    return NXConvertRGBAToColor ( r, g, b, a);
	}
    }
    return NX_COLORWHITE;
}

- default:(const char *)defName as:(const char *)defValue
{
    if( !defName || !defName[ 0]) {
	return nil;
    }
    if( !NXWriteDefault( [self appName], defName, defValue)) {
	return nil;
    }
    return self;
}
static char buffer[ 32];
- default:(const char *)defName asInt:(int)defValue
{
    sprintf( buffer, "%d", defValue);
    return [self default:defName as:buffer];
}
- default:(const char *)defName asFloat:(float)defValue
{
    sprintf( buffer, "%f", defValue);
    return [self default:defName as:buffer];
}
- default:(const char *)defName asBOOL:(BOOL)defValue
{
    return [self default:defName as:defValue ? "YES" : "NO"];
}
- default:(const char *)defName asColor:(NXColor)defValue
{
    float r, g, b, a;
    NXConvertColorToRGBA( defValue, &r, &g, &b, &a);
    sprintf( buffer, "%f %f %f %f", r, g, b, a);
    return [self default:defName as:buffer];
}

-(BOOL)cmpDefault:(const char *)defName with:(const char *)cmp
{
    return !strcmp( [self default:defName], cmp);
}
-(BOOL)cmpDefault:(const char *)defName caseWith:(const char *)cmp
{
    return !strcasecmp( [self default:defName], cmp);
}

@end
