#import "TimeMonColors.h"
#import "AppDefaults.h"
#import <appkit/TextFieldCell.h>
#import <appkit/color.h>
#import <appkit/Text.h>
#import <appkit/NXColorPanel.h>
#import <string.h>
#import "TimeMonWraps.h"

@interface TextFieldCell (TimeMonTextFieldCell)
- setColors:(NXColor)color;
@end
@implementation TimeMonColors
- initFrame:(const NXRect *)frameRect
{
    self=[super initFrame:frameRect mode:0 cellClass:[TextFieldCell class] numRows:1 numCols:4];
    if( self) {
	NXSize size={ 1, frameRect->size.height};
	[self setIntercell:&size];
	size.width=((int)frameRect->size.width-3)/4;
	[self setCellSize:&size];
	[self setAutosizeCells:YES];
	[self setFlipped:NO];
    }
    return self;
}
static const char *titles[]={
    "Idle", "Nice", "User", "System",
};
- readColors
{
    char buf[ 32];
    int i;
    for( i=0; i<4; i++) {
	TextFieldCell *tCell=[self cellAt:0 :i];
	[tCell setStringValue:titles[ i]];
	sprintf( buf, "%sColor", titles[ i]);
	[tCell setColors:[NXApp defaultAsColor:buf]];
	[tCell setBezeled:YES];
	[tCell setAlignment:NX_CENTERED];
    }
    return self;
}
- acceptColor:(NXColor)color atPoint:(NXPoint *)aPoint
{
    char buf[ 32];
    int row, col;
    NXPoint loc=*aPoint;
    TextFieldCell *tCell;
    [self convertPoint:&loc fromView:nil];
    [self getRow:&row andCol:&col forPoint:&loc];
    tCell=[self cellAt:row :col];
    [tCell setColors:color];
    [self lockFocus];
    [self drawCellInside:tCell];
    [self unlockFocus];
    [window flushWindow];
    sprintf( buf, "%sColor", titles[ col]);
    [NXApp default:buf asColor:color];
    [[NXApp delegate] display];
    return self;
}
- mouseDown:(NXEvent *)e
{
    NXPoint loc=e->location;
    int row, col;
    NXColor color;
    [self convertPoint:&loc fromView:nil];
    [self getRow:&row andCol:&col forPoint:&loc];
    color=[[self cellAt:row :col] backgroundColor];
    if( e->data.mouse.click>1) {
	[[[NXColorPanel new] setColor:color] orderFront:nil];
    } else {
	NXEvent theEvent=*e;
	int mask=[window addToEventMask:(NX_MOUSEUPMASK|NX_MOUSEDRAGGEDMASK)];
	e=[NXApp getNextEvent:(NX_MOUSEUPMASK|NX_MOUSEDRAGGEDMASK)];
	[window setEventMask:mask];
	if( e && e->type==NX_MOUSEDRAGGED) {
	    [NXColorPanel dragColor:color withEvent:&theEvent fromView:self];
	}
    }
    return self;
}
@end
@implementation TextFieldCell (TimeMonTextFieldCell)
- setColors:(NXColor)color
{
    float gray;
    float r, g, b;
    color=NXChangeAlphaComponent( color, NX_NOALPHA);
    gray=NXGrayComponent( color);
    [self setTextGray:gray<0.3 ? NX_WHITE : NX_BLACK];
    [self setBackgroundColor:color];
    [self setBackgroundGray:gray];
    NXConvertColorToRGBA( color, &r, &g, &b, NULL);
    setColor( (char *)[self stringValue], r, g, b);
    return self;
}
@end
